# -*- coding: utf-8 -*-

from ecl.tests.functional import base


class TestSubnet(base.BaseFunctionalTest):

    @classmethod
    def setUpClass(cls):
        super(TestSubnet, cls).setUpClass()
        cls.one_network = None
        networks = cls.conn.network.networks()
        for nw in networks:
            if nw.name == "test-network":
                cls.one_network = nw
                break
        cls.one_subnet = None
        subnets = cls.conn.network.subnets()
        for sn in subnets:
            if sn.name == "test_subnet":
                cls.one_subnet = sn
                break

    def test_find(self):
        sot = self.conn.network.find_subnet(self.one_subnet.name)
        self.assertEqual(self.one_subnet.id, sot.id)

    def test_get(self):
        sot = self.conn.network.get_subnet(self.one_subnet.id)
        self.assertEqual(self.one_subnet.name, sot.name)
        self.assertEqual(self.one_subnet.id, sot.id)
        self.assertEqual(self.one_subnet.dns_nameservers, sot.dns_nameservers)
        self.assertEqual(self.one_subnet.cidr, sot.cidr)
        self.assertEqual(self.one_subnet.allocation_pools, sot.allocation_pools)
        self.assertEqual(self.one_subnet.ip_version, sot.ip_version)
        self.assertEqual(self.one_subnet.host_routes, sot.host_routes)
        self.assertEqual(self.one_subnet.gateway_ip, sot.gateway_ip)
        self.assertEqual(self.one_subnet.enable_dhcp, sot.enable_dhcp)

    def test_list(self):
        names = [o.name for o in self.conn.network.subnets()]
        self.assertIn(self.one_subnet.name, names)

    def test_create(self):
        cidr = "172.16.49.0/24"
        name = "test_subnet"
        sot = self.conn.network.create_subnet(network_id=self.one_network.id,
                                              name=name, cidr=cidr)
        self.assertEqual(sot.cidr, cidr)
        self.assertEqual(sot.name, name)

    def test_update(self):
        test_description = "test subnet update (edited)"
        sot = self.conn.network.update_subnet(self.one_subnet.id,
                                              description=test_description)
        self.assertEqual(test_description, sot.description)

    def test_delete(self):
        name = "test_subnet"
        sot = self.conn.network.find_subnet(name)
        self.conn.network.delete_subnet(sot.id)



# -*- coding: utf-8 -*-

from ecl.connectivity import connectivity_service
from ecl import resource2


class ServerSegment(resource2.Resource):

    resource_key = None  # Get is not allowed.
    resources_key = 'server_segments'

    base_path = '/ec_services/%(service_id)s/server_segments'
    service = connectivity_service.ConnectivityService()

    # capabilities
    allow_get = False
    allow_list = True

    # _query_mapping = resource2.QueryParameters("mcic_id", "cic_id")

    # Properties
    #: server segment number.
    server_segment_nbr = resource2.Body('server_segment_nbr',
                                        type=int,
                                        alternate_id=True)
    #: server segment name.
    server_segment_name = resource2.Body('server_segment_name')



# -*- coding: utf-8 -*-

from ecl.block_store import block_store_service
from ecl import resource


class Extension(resource.Resource):
    resource_key = "extension"
    resources_key = "extensions"
    base_path = "/extensions"
    service = block_store_service.BlockStoreService()

    # capabilities
    allow_list = True

    # Properties
    #: A updated representing this Extension.
    updated = resource.prop("updated")
    #: Name of the extension.
    name = resource.prop("name")
    #: A list of links.
    links = resource.prop("links", type=list)
    #: Namespace of the extension.
    namespace = resource.prop("namespace")
    #: Alias of the extension.
    alias = resource.prop("alias")
    #: Description of the extension.
    description = resource.prop("description")




# -*- coding: utf-8 -*-

import six

from ecl.tests.functional import base


class TestExtension(base.BaseFunctionalTest):

    def test_list(self):
        sots = self.conn.block_store.extensions()
        self.assertGreaterEqual(len(sots), 0)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from ecl.database.v1 import instance as _instance
from ecl.database.v1 import user as _user
from ecl.database.v1 import database as _database
from ecl.database.v1 import flavor as _flavor
from ecl.database.v1 import datastore as _datastore

from ecl import proxy2
from ecl import resource2


class Proxy(proxy2.BaseProxy):

    def instances(self, **query):
        """Retrieve a list of instances

        :param kwargs \*\*query: Optional query parameters to be sent to limit
                                 the instances being returned.
        :returns: A list of database instances.
        """
        instance = _instance.Instance
        return list(self._list(instance, paginated=False, **query))

    def create_instance(self,
                        name,
                        flavor_id,
                        volume,
                        datastore,
                        nics,
                        databases=None,
                        users=None,
                        availability_zone=None,
                        backup_window=None,
                        backup_retention_period=None,
                        restore_point=None,
                        maintenance_window=None,
                        **attrs):
        """Create a new instance from attributes

        :param string name: Name of instance
        :param string flavor_id: Flavor ID of server
        :param dict volue: Volume configuration dict which has volume_size inside
        :param databases: Database definition
                list to initialize database on creating instance
        :param users: List of users to connect to defined databases
        :param dict datastore: Datastore name and version of instance
        :param dict nics: Network difinition of instance
        :param availability_zone: Availability zone for instance
        :param backup_window: Backup window time range
        :param backup_retention_period: Number of the day to retain backup
        :param maintenance_window: Maintenance window time range by
                the day of the week and from/to time
        :param kwargs attrs: Keyword arguments which will be used to create
                             a :class:`~ecl.datagase.v1.instance.Instance`,
                             comprised of the properties on the Instance class.

        :returns: The results of instance creation
        :rtype: :class:`~ecl.compute.v1.instance.Instance`
        """
        attrs.update({"name": name})
        attrs.update({"flavorRef": flavor_id})
        attrs.update({"volume": volume})
        attrs.update({"datastore": datastore})
        attrs.update({"nics": nics})

        if databases:
            attrs.update({"databases": databases})
        if users:
            attrs.update({"users": users})
        if availability_zone:
            attrs.update({"availability_zone": availability_zone})
        if backup_window:
            attrs.update({"backup_window": backup_window})
        if backup_retention_period or backup_retention_period == 0:
            attrs.update({"backup_retention_period": backup_retention_period})
        if restore_point:
            attrs.update({"restorePoint": restore_point})
        if maintenance_window:
            attrs.update({"maintenance_window": maintenance_window})

        return self._create(_instance.Instance, **attrs)

    def delete_instance(self, instance, ignore_missing=False):
        """Delete a instance

        :param instance: The value can be either the ID of a server or a
                         :class:`~ecl.database.v1.instance.Instance` instance.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the server does not exist.
                    When set to ``True``, no exception will be set when
                    attempting to delete a nonexistent instance
        :param bool force: When set to ``True``, the instance deletion will be
                           forced immediatly.

        :returns: ``None``
        """
        self._delete(_instance.Instance, instance, ignore_missing=ignore_missing)

    def find_instance(self, name_or_id, ignore_missing=False):
        """Find a single instance

        :param name_or_id: The name or ID of a server.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the resource does not exist.
                    When set to ``True``, None will be returned when
                    attempting to find a nonexistent resource.
        :returns: One :class:`~ecl.database.v1.instance.Instance` or None
        """
        return self._find(_instance.Instance, name_or_id,
                          ignore_missing=ignore_missing)

    def get_instance(self, instance):
        """Get a single instance

        :param instance: The value can be the ID of a instance or a
                       :class:`~ecl.database.v1.instance.Instance` instance.

        :returns: One :class:`~ecl.database.v1.instance.Instance`
        :raises: :class:`~ecl.exceptions.ResourceNotFound`
                 when no resource can be found.
        """
        return self._get(_instance.Instance, instance)

    def wait_for_server(self, instance, status='ACTIVE', failures=['ERROR'],
                        interval=2, wait=120):
        return resource2.wait_for_status(self.session, instance, status,
                                         failures, interval, wait)

    # def find_flavor(self, name_or_id, ignore_missing=False):
    #     """Find a single flavor

    #     :param name_or_id: The name or ID of a flavor.
    #     :param bool ignore_missing: When set to ``False``
    #                 :class:`~ecl.exceptions.ResourceNotFound` will be
    #                 raised when the resource does not exist.
    #                 When set to ``True``, None will be returned when
    #                 attempting to find a nonexistent resource.
    #     :returns: One :class:`~ecl.database.v1.flavor.Flavor` or None
    #     """
    #     return self._find(_flavor.Flavor, name_or_id,
    #                       ignore_missing=ignore_missing)

    # def get_flavor(self, flavor):
    #     """Get a single flavor

    #     :param flavor: The value can be the ID of a flavor or a
    #                    :class:`~ecl.database.v1.flavor.Flavor` instance.

    #     :returns: One :class:`~ecl.database.v1.flavor.Flavor`
    #     :raises: :class:`~ecl.exceptions.ResourceNotFound`
    #              when no resource can be found.
    #     """
    #     return self._get(_flavor.Flavor, flavor)

    def flavors(self):
        """Return a list of flavors
        :returns: A list of flavor objects
        """
        return list(self._list(_flavor.Flavor, paginated=False))

    def datastores(self):
        """Return a list of datastores
        :returns: A list of datastore objects
        """
        return list(self._list(_datastore.Datastore, paginated=False))

    def users(self, instance_id, **query):
        """Retrieve a list of users assciated with instance

        :param instance_id: Instance id to find users
        :param kwargs \*\*query: Optional query parameters to be sent to limit
            the instances being returned.  Available parameters include:
        :returns: A list of database instances.
        """
        user = _user.User
        return list(self._list(user, paginated=False, instance_id=instance_id, **query))

    def create_user(self, instance_id, name, password, databases=None):
        """Create a new user from attributes

        :param string instance_id: ID of instance to assciate creating user
        :param string name: Name of user
        :param string password: Password of user
        :param databases: Database list of user to grant

        :returns: ``None``
        """
        attrs = {"name": name,
                 "password": password}
        if databases:
            attrs.update({"databases": databases})
        user = _user.User()
        return user.create(self.session, instance_id, **attrs)

    def delete_user(self, instance_id, user, ignore_missing=False):
        """Delete a user

        :param instance_id: The value can be either the ID of a server or a
                           :class:`~ecl.database.v1.instance.Instance` instance.
        :param string user: Name of user.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the resource does not exist.
                    When set to ``True``, no exception will be set when
                    attempting to delete a nonexistent user

        :returns: ``None``
        """
        self._delete(_user.User, user, instance_id=instance_id,
                     ignore_missing=ignore_missing)

    def find_user(self, instance_id, name_or_id, ignore_missing=False):
        """Find a single user

        :param name_or_id: The name or ID of a user.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the resource does not exist.
                    When set to ``True``, None will be returned when
                    attempting to find a nonexistent resource.
        :returns: One :class:`~ecl.database.v1.user.User` or None
        """
        return self._find(_user.User, name_or_id,
                          instance_id=instance_id,
                          ignore_missing=ignore_missing)

    def get_user(self, instance_id, user):
        """Show a user

        :param instance_id: The value can be either the ID of a server or a
                           :class:`~ecl.database.v1.instance.Instance` instance.
        :param string user: Name of user.
        :returns: One user.
        :rtype: :class:`~ecl.compute.v1.user.User`
        """
        return self._get(_user.User, user, instance_id=instance_id)

    def grant_user(self, instance_id, user_name, databases):
        """Grants database access privilege to user in DB Instance.

        :param instance_id: The value can be either the ID of a server or a
                           :class:`~ecl.database.v1.instance.Instance` instance.
        :param string user: Name of user.
        :param array databases: Database names to grant access privilege.
        :returns: ``None``
        """
        user = _user.User()
        return user.grant(self.session, instance_id, user_name, databases)

    def revoke_user(self, instance_id, user_name, database):
        """Revoke the access privilege of user from database in DB Instance.

        :param instance_id: The value can be either the ID of a server or a
                           :class:`~ecl.database.v1.instance.Instance` instance.
        :param string user: Name of user.
        :param string database: Database name to revoke access privilege.
        :returns: ``None``
        """
        user = _user.User()
        return user.revoke(self.session, instance_id, user_name, database)

    def databases(self, instance_id, **query):
        """Retrieve a list of databases assciated with instance

        :param instance_id: Instance id to find databases
        :param kwargs \*\*query: Optional query parameters to be sent to limit
            the instances being returned.  Available parameters include:
        :returns: A list of database instances.
        """
        database = _database.Database
        return list(self._list(database, paginated=False,
                               instance_id=instance_id, **query))

    def create_database(self, instance_id, name, charset=None, collate=None):
        """Create a new database from attributes

        :param string instance_id: ID of instance to assciate creating database
        :param string name: Name of database

        :returns: ``None``
        """
        attrs = {"name": name}
        if charset:
            attrs.update({"character_set": charset})
        if collate:
            attrs.update({"collate": collate})
        database = _database.Database()
        return database.create(self.session, instance_id, **attrs)

    def delete_database(self, instance_id, database, ignore_missing=False):
        """Delete a database

        :param instance_id: The value can be either the ID of a server or a
                           :class:`~ecl.database.v1.instance.Instance` instance.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the resource does not exist.
                    When set to ``True``, no exception will be set when
                    attempting to delete a nonexistent database

        :returns: ``None``
        """
        self._delete(_database.Database, database, instance_id=instance_id,
                     ignore_missing=ignore_missing)

    def find_database(self, instance_id, name_or_id, ignore_missing=False):
        """Find a single database

        :param name_or_id: The name or ID of a database.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the resource does not exist.
                    When set to ``True``, None will be returned when
                    attempting to find a nonexistent resource.
        :returns: One :class:`~ecl.database.v1.database.Database` or None
        """
        return self._find(_database.Database, name_or_id,
                          instance_id=instance_id,
                          ignore_missing=ignore_missing)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from ecl.identity.v3 import _proxy
from ecl.identity.v3 import credential
from ecl.identity.v3 import domain
from ecl.identity.v3 import endpoint
from ecl.identity.v3 import group
from ecl.identity.v3 import policy
from ecl.identity.v3 import project
from ecl.identity.v3 import region
from ecl.identity.v3 import service
from ecl.identity.v3 import trust
from ecl.identity.v3 import user
from ecl.tests.unit import test_proxy_base


class TestIdentityProxy(test_proxy_base.TestProxyBase):
    def setUp(self):
        super(TestIdentityProxy, self).setUp()
        self.proxy = _proxy.Proxy(self.session)

    def test_credential_create_attrs(self):
        self.verify_create(self.proxy.create_credential,
                           credential.Credential)

    def test_credential_delete(self):
        self.verify_delete(self.proxy.delete_credential,
                           credential.Credential, False)

    def test_credential_delete_ignore(self):
        self.verify_delete(self.proxy.delete_credential,
                           credential.Credential, True)

    def test_credential_find(self):
        self.verify_find(self.proxy.find_credential, credential.Credential)

    def test_credential_get(self):
        self.verify_get(self.proxy.get_credential, credential.Credential)

    def test_credentials(self):
        self.verify_list(self.proxy.credentials, credential.Credential,
                         paginated=False)

    def test_credential_update(self):
        self.verify_update(self.proxy.update_credential, credential.Credential)

    def test_domain_create_attrs(self):
        self.verify_create(self.proxy.create_domain, domain.Domain)

    def test_domain_delete(self):
        self.verify_delete(self.proxy.delete_domain, domain.Domain, False)

    def test_domain_delete_ignore(self):
        self.verify_delete(self.proxy.delete_domain, domain.Domain, True)

    def test_domain_find(self):
        self.verify_find(self.proxy.find_domain, domain.Domain)

    def test_domain_get(self):
        self.verify_get(self.proxy.get_domain, domain.Domain)

    def test_domains(self):
        self.verify_list(self.proxy.domains, domain.Domain, paginated=False)

    def test_domain_update(self):
        self.verify_update(self.proxy.update_domain, domain.Domain)

    def test_endpoint_create_attrs(self):
        self.verify_create(self.proxy.create_endpoint, endpoint.Endpoint)

    def test_endpoint_delete(self):
        self.verify_delete(self.proxy.delete_endpoint,
                           endpoint.Endpoint, False)

    def test_endpoint_delete_ignore(self):
        self.verify_delete(self.proxy.delete_endpoint,
                           endpoint.Endpoint, True)

    def test_endpoint_find(self):
        self.verify_find(self.proxy.find_endpoint, endpoint.Endpoint)

    def test_endpoint_get(self):
        self.verify_get(self.proxy.get_endpoint, endpoint.Endpoint)

    def test_endpoints(self):
        self.verify_list(self.proxy.endpoints, endpoint.Endpoint,
                         paginated=False)

    def test_endpoint_update(self):
        self.verify_update(self.proxy.update_endpoint, endpoint.Endpoint)

    def test_group_create_attrs(self):
        self.verify_create(self.proxy.create_group, group.Group)

    def test_group_delete(self):
        self.verify_delete(self.proxy.delete_group, group.Group, False)

    def test_group_delete_ignore(self):
        self.verify_delete(self.proxy.delete_group, group.Group, True)

    def test_group_find(self):
        self.verify_find(self.proxy.find_group, group.Group)

    def test_group_get(self):
        self.verify_get(self.proxy.get_group, group.Group)

    def test_groups(self):
        self.verify_list(self.proxy.groups, group.Group, paginated=False)

    def test_group_update(self):
        self.verify_update(self.proxy.update_group, group.Group)

    def test_policy_create_attrs(self):
        self.verify_create(self.proxy.create_policy, policy.Policy)

    def test_policy_delete(self):
        self.verify_delete(self.proxy.delete_policy, policy.Policy, False)

    def test_policy_delete_ignore(self):
        self.verify_delete(self.proxy.delete_policy, policy.Policy, True)

    def test_policy_find(self):
        self.verify_find(self.proxy.find_policy, policy.Policy)

    def test_policy_get(self):
        self.verify_get(self.proxy.get_policy, policy.Policy)

    def test_policies(self):
        self.verify_list(self.proxy.policies, policy.Policy, paginated=False)

    def test_policy_update(self):
        self.verify_update(self.proxy.update_policy, policy.Policy)

    def test_project_create_attrs(self):
        self.verify_create(self.proxy.create_project, project.Project)

    def test_project_delete(self):
        self.verify_delete(self.proxy.delete_project, project.Project, False)

    def test_project_delete_ignore(self):
        self.verify_delete(self.proxy.delete_project, project.Project, True)

    def test_project_find(self):
        self.verify_find(self.proxy.find_project, project.Project)

    def test_project_get(self):
        self.verify_get(self.proxy.get_project, project.Project)

    def test_projects(self):
        self.verify_list(self.proxy.projects, project.Project, paginated=False)

    def test_project_update(self):
        self.verify_update(self.proxy.update_project, project.Project)

    def test_service_create_attrs(self):
        self.verify_create(self.proxy.create_service, service.Service)

    def test_service_delete(self):
        self.verify_delete(self.proxy.delete_service, service.Service, False)

    def test_service_delete_ignore(self):
        self.verify_delete(self.proxy.delete_service, service.Service, True)

    def test_service_find(self):
        self.verify_find(self.proxy.find_service, service.Service)

    def test_service_get(self):
        self.verify_get(self.proxy.get_service, service.Service)

    def test_services(self):
        self.verify_list(self.proxy.services, service.Service, paginated=False)

    def test_service_update(self):
        self.verify_update(self.proxy.update_service, service.Service)

    def test_user_create_attrs(self):
        self.verify_create(self.proxy.create_user, user.User)

    def test_user_delete(self):
        self.verify_delete(self.proxy.delete_user, user.User, False)

    def test_user_delete_ignore(self):
        self.verify_delete(self.proxy.delete_user, user.User, True)

    def test_user_find(self):
        self.verify_find(self.proxy.find_user, user.User)

    def test_user_get(self):
        self.verify_get(self.proxy.get_user, user.User)

    def test_users(self):
        self.verify_list(self.proxy.users, user.User, paginated=False)

    def test_user_update(self):
        self.verify_update(self.proxy.update_user, user.User)

    def test_trust_create_attrs(self):
        self.verify_create(self.proxy.create_trust, trust.Trust)

    def test_trust_delete(self):
        self.verify_delete(self.proxy.delete_trust, trust.Trust, False)

    def test_trust_delete_ignore(self):
        self.verify_delete(self.proxy.delete_trust, trust.Trust, True)

    def test_trust_find(self):
        self.verify_find(self.proxy.find_trust, trust.Trust)

    def test_trust_get(self):
        self.verify_get(self.proxy.get_trust, trust.Trust)

    def test_trusts(self):
        self.verify_list(self.proxy.trusts, trust.Trust, paginated=False)

    def test_region_create_attrs(self):
        self.verify_create(self.proxy.create_region, region.Region)

    def test_region_delete(self):
        self.verify_delete(self.proxy.delete_region, region.Region, False)

    def test_region_delete_ignore(self):
        self.verify_delete(self.proxy.delete_region, region.Region, True)

    def test_region_find(self):
        self.verify_find(self.proxy.find_region, region.Region)

    def test_region_get(self):
        self.verify_get(self.proxy.get_region, region.Region)

    def test_regions(self):
        self.verify_list(self.proxy.regions, region.Region, paginated=False)

    def test_region_update(self):
        self.verify_update(self.proxy.update_region, region.Region)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


import six
from ecl.tests.functional import base


class TestLimits(base.BaseFunctionalTest):

    def test_01_Get_Limits(self):
        limit = self.conn.baremetal.get_limits()
        print limit
        self.assertIsInstance(limit.absolute, dict)
        self.assertIsInstance(limit.rate, list)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from ecl.image.v2 import image as _image
from ecl.image.v2 import image_copy as _copy
from ecl.image.v2 import member as _member
from ecl.image.v2 import license as _license
from ecl.image.v2 import tag as _tag
from ecl.image.v2 import schema as _schema
from ecl import proxy2
from ecl import resource2


class Proxy(proxy2.BaseProxy):

    def create_image(self, **attrs):
        """Creates a virtual machine (VM) image.

        :param dict attrs: Keyword arguments which will be used to create
                           a :class:`~ecl.image.v2.image.Image`,
                           comprised of the properties on the Image class.

        :returns: The results of image creation
        :rtype: :class:`~ecl.image.v2.image.Image`
        """
        img = self._create(_image.Image, **attrs)
        return img

    def upload_image(self, image_id, image_data):
        """
        Uploads binary image data.

        :param image_id: An identifier for the image.
        :param image_data: raw binary data that represents the actual virtual disk.
        :return: ``None``
        """
        img = _image.Image()
        return img.upload(self.session, image_id, image_data)

    def download_image(self, image_id):
        """Download an image

        :param image: The value can be either the ID of an image or a
                      :class:`~ecl.image.v2.image.Image` instance.

        :returns: The bytes comprising the given Image.
        """
        img = _image.Image()
        return img.download(self.session, image_id)

    def delete_image(self, image, ignore_missing=False):
        """Delete an image

        :param image: The value can be either the ID of an image or a
                      :class:`~ecl.image.v2.image.Image` instance.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the image does not exist.
                    When set to ``True``, no exception will be set when
                    attempting to delete a nonexistent image.

        :returns: ``None``
        """
        self._delete(_image.Image, image, ignore_missing=ignore_missing)

    def find_image(self, name_or_id, ignore_missing=False):
        """Find a single image

        :param name_or_id: The name or ID of a image.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the resource does not exist.
                    When set to ``True``, None will be returned when
                    attempting to find a nonexistent resource.
        :returns: One :class:`~ecl.image.v2.image.Image` or None
        """

        return self._find(_image.Image, name_or_id,
                          ignore_missing=ignore_missing)

    def get_image(self, image):
        """Get a single image

        :param image: The value can be the ID of a image or a
                      :class:`~ecl.image.v2.image.Image` instance.

        :returns: One :class:`~ecl.image.v2.image.Image`
        :raises: :class:`~ecl.exceptions.ResourceNotFound`
                 when no resource can be found.
        """
        img = _image.Image()
        return img.get(self.session, image)

    def images(self, **query):
        """Return a generator of images

        :param kwargs \*\*query: Optional query parameters to be sent to limit
                                 the resources being returned.

        :returns: A generator of image objects
        :rtype: :class:`~ecl.image.v2.image.Image`
        """
        if 'limit' not in query.keys():
            query["limit"] = 1000
        return list(self._list(_image.Image, **query))

    def update_image(self, image_id, image_data):
        """
        Update a image.

        :param image_id: ID of a image.
        :param image_data: List of attributes(dict) to update on the image.
        :return: :class:`~ecl.image.v2.image.Image`
        """
        img = _image.Image()
        return img.update(self.session, image_id, image_data)

    def copy_image(self, image_id, tenant_id_dst):
        """
        Copy image to a specified region.

        :param image_id: An identifier for the image.
        :param tenant_id_dst: An identifier for the image tenant for destination region.
        :return: :class:`~ecl.image.v2.image_copy.ImageCopy`
        """
        copy = _copy.ImageCopy()
        return copy.copy_image(self.session, image_id, tenant_id_dst)

    def cancel_copy_image(self, job_id):
        """
        Lists details for image copy jobs.

        :param job_id: JOB_ID that has been paid out in the Copy API.
        :return: ``None``
        """
        copy = _copy.ImageCopy()
        return copy.cancel_copy_image(self.session, job_id)

    def list_image_copy_jobs(self):
        """
        Lists details for image copy jobs.

        :return: A generator of ImageCopy instances.
        :rtype: :class:`~ecl.image.v2.image_copy.ImageCopy`
        """
        copy = _copy.ImageCopy()
        return copy.list_image_copy_jobs(self.session)

    def get_image_copy_job(self, job_id):
        """
        Get details for a specified image copy job.

        :param job_id: JOB_ID that has been paid out in the Copy API.
        :return: :class:`~ecl.image.v2.image_copy.ImageCopy`
        """
        copy = _copy.ImageCopy()
        return copy.get_image_copy_job(self.session, job_id)

    def add_tag(self, image, tag):
        """Add a tag to an image

        :param image: The value can be the ID of a image or a
                      :class:`~ecl.image.v2.image.Image` instance
                      that the member will be created for.
        :param str tag: The tag to be added

        :returns: None
        """
        image = self._get_resource(_image.Image, image)
        image.add_tag(self.session, tag)

    def remove_tag(self, image, tag):
        """Remove a tag to an image

        :param image: The value can be the ID of a image or a
                      :class:`~ecl.image.v2.image.Image` instance
                      that the member will be created for.
        :param str tag: The tag to be removed

        :returns: None
        """
        image = self._get_resource(_image.Image, image)
        image.remove_tag(self.session, tag)

    def create_member(self, image, **attrs):
        """Create a new member from attributes

        :param image: The value can be the ID of a image or a
                      :class:`~ecl.image.v2.image.Image` instance
                      that the member will be created for.
        :param dict attrs: Keyword arguments which will be used to create
                           a :class:`~ecl.image.v2.member.Member`,
                           comprised of the properties on the Member class.

        :returns: The results of member creation
        :rtype: :class:`~ecl.image.v2.member.Member`
        """
        image_id = resource2.Resource._get_id(image)
        return self._create(_member.Member, image_id=image_id, **attrs)

    def delete_member(self, image, member, ignore_missing=False):
        """Delete a member

        :param member: The value can be either the ID of a member or a
                       :class:`~ecl.image.v2.member.Member` instance.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the member does not exist.
                    When set to ``True``, no exception will be set when
                    attempting to delete a nonexistent member.

        :returns: ``None``
        """
        image_id = resource2.Resource._get_id(image)
        member_id = resource2.Resource._get_id(member)
        self._delete(_member.Member, member_id, image_id=image_id,
                     ignore_missing=ignore_missing)

    def get_member(self, image, member):
        """Get a single member on an image

        :param member: The value can be the ID of a member or a
                       :class:`~ecl.image.v2.member.Member` instance.
        :param image: This is the image that the member belongs to.
                      The value can be the ID of a image or a
                      :class:`~ecl.image.v2.image.Image` instance.
        :returns: One :class:`~ecl.image.v2.member.Member`
        :raises: :class:`~ecl.exceptions.ResourceNotFound`
                 when no resource can be found.
        """
        member_id = resource2.Resource._get_id(member)
        image_id = resource2.Resource._get_id(image)
        return self._get(_member.Member, member_id, image_id=image_id)

    def members(self, image):
        """Return a generator of members

        :param image: This is the image that the member belongs to,
                      the value can be the ID of a image or a
                      :class:`~ecl.image.v2.image.Image` instance.

        :returns: A generator of member objects
        :rtype: :class:`~ecl.image.v2.member.Member`
        """
        image_id = resource2.Resource._get_id(image)
        return self._list(_member.Member, paginated=False,
                          image_id=image_id)

    def update_member(self, image_id, member_id, status):
        """
        Update the member of an image.

        :param image_id: ID of a member.
        :param member_id: ID of the image that the member belongs to.
        :param status: The status of this image member.
        :returns: The updated member
        :rtype: :class:`~ecl.image.v2.member.Member`
        """
        member = _member.Member()
        return member.update(self.session, image_id, member_id, status)

    def licenses(self):
        """
        List image license switch types.

        :returns: A generator of license objects
        :rtype: :class:`~ecl.image.v2.license.License`
        """
        license = _license.License()
        return license.list(self.session)

    def add_tag(self, image_id, tag_to_add):
        """
        Adds a specified tag to a specified image.

        :param image_id: Image ID stored through the image API.
                        Typically a UUID.
        :param tag: Image tag to add.
        :return: ``None``
        """
        tag = _tag.Tag()
        return tag.add_tag(self.session, image_id, tag_to_add)

    def delete_tag(self, image_id, tag_to_delete):
        """
        Deletes a specified tag from a specified image.

        :param image_id: Image ID stored through the image API.
                        Typically a UUID.
        :param tag_to_delete: Image tag to delete.
        :return: ``None``
        """
        tag = _tag.Tag()
        return tag.delete_tag(self.session, image_id, tag_to_delete)

    def get_image_schema(self):
        """
        Gets a JSON schema document that represents an image entity.

        :return: :class:`~ecl.image.v2.scheme.Scheme`
        """
        schema = _schema.Scheme()
        return schema.get_image_schema(self.session)

    def get_images_schema(self):
        """
        Gets a JSON schema document that represents an images entity.

        :return: :class:`~ecl.image.v2.scheme.Scheme`
        """
        schema = _schema.Scheme()
        return schema.get_images_schema(self.session)

    def get_image_member_schema(self):
        """
        Gets a JSON schema document that represents an image member
        entity.

        :return: :class:`~ecl.image.v2.scheme.Scheme`
        """
        schema = _schema.Scheme()
        return schema.get_image_member_schema(self.session)

    def get_metadata_namespace_schema(self):
        """
        Gets a JSON schema document that represents a metadata
        definition namespace entity.

        :return: :class:`~ecl.image.v2.scheme.Scheme`
        """
        schema = _schema.Scheme()
        return schema.get_metadata_namespace_schema(self.session)

    def get_metadata_object_schema(self):
        """
        Gets a JSON schema document that represents a metadata
        definition object entity.

        :return: :class:`~ecl.image.v2.scheme.Scheme`
        """
        schema = _schema.Scheme()
        return schema.get_metadata_object_schema(self.session)

    def get_metadata_property_schema(self):
        """
        Gets a JSON schema document that represents a metadata
        definition property entity.

        :return: :class:`~ecl.image.v2.scheme.Scheme`
        """
        schema = _schema.Scheme()
        return schema.get_metadata_property_schema(self.session)

    def get_metadata_resource_type_schema(self):
        """
        Gets a JSON schema document that represents a metadata
        definition namespace resource type association entity.

        :return: :class:`~ecl.image.v2.scheme.Scheme`
        """
        schema = _schema.Scheme()
        return schema.get_metadata_resource_type_schema(self.session)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import six

from ecl.tests.functional import base


class TestType(base.BaseFunctionalTest):

    def test_list(self):
        sots = self.conn.block_store.types()
        for sot in sots:
            self.assertIsInstance(sot.id, six.string_types)

    def test_find(self):
        sot = self.conn.block_store.find_volume_type("nfsdriver")
        print sot
        


# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from ecl.identity import identity_service
from ecl import resource


class User(resource.Resource):
    resource_key = 'user'
    resources_key = 'users'
    base_path = '/users'
    service = identity_service.AdminService()

    # capabilities
    allow_create = True
    allow_retrieve = True
    allow_update = True
    allow_delete = True
    allow_list = True

    # Properties
    #: The email of this user. *Type: string*
    email = resource.prop('email')
    #: Setting this value to ``False`` prevents the user from authenticating or
    #: receiving authorization. Additionally, all pre-existing tokens held by
    #: the user are immediately invalidated. Re-enabling a user does not
    #: re-enable pre-existing tokens. *Type: bool*
    is_enabled = resource.prop('enabled', type=bool)
    #: The name of this user. *Type: string*
    name = resource.prop('name')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from ecl.object_store.v1 import account as _account
from ecl.object_store.v1 import container as _container
from ecl.object_store.v1 import obj as _obj
from ecl import proxy


class Proxy(proxy.BaseProxy):

    def get_account_metadata(self):
        """Get metadata for this account.

        :rtype:
            :class:`~ecl.object_store.v1.account.Account`
        """
        return self._head(_account.Account)

    def set_account_metadata(self, **metadata):
        """Set metadata for this account.

        :param kwargs metadata: Key/value pairs to be set as metadata
                                on the container. Custom  metadata can be set.
                                Custom metadata are keys and values defined
                                by the user.
        """
        account = self._get_resource(_account.Account, None)
        account.set_metadata(self.session, metadata)

    def delete_account_metadata(self, keys):
        """Delete metadata for this account.

        :param keys: The keys of metadata to be deleted.
        """
        account = self._get_resource(_account.Account, None)
        account.delete_metadata(self.session, keys)

    def containers(self, **query):
        """Obtain Container objects for this account.

        :param kwargs query: Optional query parameters to be sent to limit
                                 the resources being returned.

        :rtype: A generator of
            :class:`~ecl.object_store.v1.container.Container` objects.
        """
        return list(_container.Container.list(self.session, **query))

    def create_container(self, **attrs):
        """Create a new container from attributes

        :param dict attrs: Keyword arguments which will be used to create
               a :class:`~ecl.object_store.v1.container.Container`,
               comprised of the properties on the Container class.

        :returns: The results of container creation
        :rtype: :class:`~ecl.object_store.v1.container.Container`
        """
        return self._create(_container.Container, **attrs)

    def delete_container(self, container, ignore_missing=False):
        """Delete a container

        :param container: The value can be either the name of a container or a
                      :class:`~ecl.object_store.v1.container.Container`
                      instance.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the container does not exist.
                    When set to ``True``, no exception will be set when
                    attempting to delete a nonexistent server.

        :returns: ``None``
        """
        self._delete(_container.Container, container,
                     ignore_missing=ignore_missing)

    def get_container_metadata(self, container):
        """Get metadata for a container

        :param container: The value can be the name of a container or a
               :class:`~ecl.object_store.v1.container.Container`
               instance.

        :returns: One :class:`~ecl.object_store.v1.container.Container`
        :raises: :class:`~ecl.exceptions.ResourceNotFound`
                 when no resource can be found.
        """
        container = self._head(_container.Container, container)
        container.count = container.object_count
        container.size = container.bytes_used
        return container

    def set_container_metadata(self, container, **metadata):
        """Set metadata for a container.

        :param container: The value can be the name of a container or a
               :class:`~ecl.object_store.v1.container.Container`
               instance.
        :param kwargs metadata: Key/value pairs to be set as metadata
                                on the container. Both custom and system
                                metadata can be set. Custom metadata are keys
                                and values defined by the user. System
                                metadata are keys defined by the Object Store
                                and values defined by the user. The system
                                metadata keys are:

                                - `content_type`
                                - `is_content_type_detected`
                                - `versions_location`
                                - `read_ACL`
                                - `write_ACL`
                                - `sync_to`
                                - `sync_key`
        """
        res = self._get_resource(_container.Container, container)
        res.set_metadata(self.session, metadata)

    def delete_container_metadata(self, container, keys):
        """Delete metadata for a container.

        :param container: The value can be the ID of a container or a
               :class:`~ecl.object_store.v1.container.Container`
               instance.
        :param keys: The keys of metadata to be deleted.
        """
        res = self._get_resource(_container.Container, container)
        res.delete_metadata(self.session, keys)

    def objects(self, container, **query):
        """Return a generator that yields the Container's objects.

        :param container: A container object or the name of a container
            that you want to retrieve objects from.
        :type container:
            :class:`~ecl.object_store.v1.container.Container`
        :param kwargs \*\*query: Optional query parameters to be sent to limit
                                 the resources being returned.

        :rtype: A generator of
            :class:`~ecl.object_store.v1.obj.Object` objects.
        """
        container = _container.Container.from_id(container)

        objs = _obj.Object.list(self.session,
                                path_args={"container": container.name},
                                **query)
        objs = list(objs)
        for i in range(len(objs)):
            objs[i].container = container.name

        return objs

    def _get_container_name(self, obj, container):
        if isinstance(obj, _obj.Object):
            if obj.container is not None:
                return obj.container
        if container is not None:
            container = _container.Container.from_id(container)
            return container.name

        raise ValueError("container must be specified")

    def get_object(self, obj, container=None):
        """Get the data associated with an object

        :param obj: The value can be the name of an object or a
                       :class:`~ecl.object_store.v1.obj.Object` instance.
        :param container: The value can be the name of a container or a
               :class:`~ecl.object_store.v1.container.Container`
               instance.

        :returns: The contents of the object.  Use the
                  :func:`~get_object_metadata`
                  method if you want an object resource.
        :raises: :class:`~ecl.exceptions.ResourceNotFound`
                 when no resource can be found.
        """
        # TODO(briancurtin): call this download_object and make sure it's
        # just returning the raw data, like download_image does
        container_name = self._get_container_name(obj, container)

        return self._get(_obj.Object, obj,
                         path_args={"container": container_name})

    def download_object(self, obj, container=None, path=None):
        """Download the data contained inside an object to disk.

        :param obj: The value can be the name of an object or a
                       :class:`~ecl.object_store.v1.obj.Object` instance.
        :param container: The value can be the name of a container or a
               :class:`~ecl.object_store.v1.container.Container`
               instance.
        :param path str: Location to write the object contents.

        :raises: :class:`~ecl.exceptions.ResourceNotFound`
                 when no resource can be found.
        """
        # TODO(briancurtin): download_object should really have the behavior
        # of get_object, and this writing to a file should not exist.
        # TODO(briancurtin): This method should probably offload the get
        # operation into another thread or something of that nature.
        with open(path, "w") as out:
            out.write(self.get_object(obj, container))

    def upload_object(self, **attrs):
        """Upload a new object from attributes

        :param dict attrs: Keyword arguments which will be used to create
               a :class:`~ecl.object_store.v1.obj.Object`,
               comprised of the properties on the Object class.
               **Required**: A `container` argument must be specified,
               which is either the ID of a container or a
               :class:`~ecl.object_store.v1.container.Container`
               instance.

        :returns: The results of object creation
        :rtype: :class:`~ecl.object_store.v1.container.Container`
        """
        container = attrs.pop("container", None)
        container_name = self._get_container_name(None, container)

        return self._create(_obj.Object,
                            path_args={"container": container_name}, **attrs)

    def copy_object(self):
        """Copy an object."""
        raise NotImplementedError

    def delete_object(self, obj, ignore_missing=False, container=None):
        """Delete an object

        :param obj: The value can be either the name of an object or a
                       :class:`~ecl.object_store.v1.container.Container`
                       instance.
        :param container: The value can be the ID of a container or a
               :class:`~ecl.object_store.v1.container.Container`
               instance.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the object does not exist.
                    When set to ``True``, no exception will be set when
                    attempting to delete a nonexistent server.

        :returns: ``None``
        """
        container_name = self._get_container_name(obj, container)

        self._delete(_obj.Object, obj, ignore_missing=ignore_missing,
                     path_args={"container": container_name})

    def get_object_metadata(self, obj, container=None):
        """Get metadata for an object.

        :param obj: The value can be the name of an object or a
                    :class:`~ecl.object_store.v1.obj.Object` instance.
        :param container: The value can be the ID of a container or a
               :class:`~ecl.object_store.v1.container.Container`
               instance.

        :returns: One :class:`~ecl.object_store.v1.obj.Object`
        :raises: :class:`~ecl.exceptions.ResourceNotFound`
                 when no resource can be found.
        """
        container_name = self._get_container_name(obj, container)

        return self._head(_obj.Object, obj,
                          path_args={"container": container_name})

    def set_object_metadata(self, obj, container=None, **metadata):
        """Set metadata for an object.

        Note: This method will do an extra HEAD call.

        :param obj: The value can be the name of an object or a
                    :class:`~ecl.object_store.v1.obj.Object` instance.
        :param container: The value can be the name of a container or a
               :class:`~ecl.object_store.v1.container.Container`
               instance.
        :param kwargs metadata: Key/value pairs to be set as metadata
                                on the container. Both custom and system
                                metadata can be set. Custom metadata are keys
                                and values defined by the user. System
                                metadata are keys defined by the Object Store
                                and values defined by the user. The system
                                metadata keys are:

                                - `content_type`
                                - `content_encoding`
                                - `content_disposition`
                                - `delete_after`
                                - `delete_at`
                                - `is_content_type_detected`
        """
        container_name = self._get_container_name(obj, container)
        res = self._get_resource(_obj.Object, obj,
                                 path_args={"container": container_name})
        res.set_metadata(self.session, metadata)

    def delete_object_metadata(self, obj, container=None, keys=None):
        """Delete metadata for an object.

        :param obj: The value can be the name of an object or a
                    :class:`~ecl.object_store.v1.obj.Object` instance.
        :param container: The value can be the ID of a container or a
               :class:`~ecl.object_store.v1.container.Container`
               instance.
        :param keys: The keys of metadata to be deleted.
        """
        container_name = self._get_container_name(obj, container)
        res = self._get_resource(_obj.Object, obj,
                                 path_args={"container": container_name})
        res.delete_metadata(self.session, keys)



# -*- coding: utf-8 -*-

from ecl.tests.functional import base


class TestImage(base.BaseFunctionalTest):

    # def test_images(self):
    #     images = list(self.conn.compute.images())
    #     self.assertGreater(len(images), 0)
    #     for image in images:
    #         self.assertIsInstance(image.id, six.string_types)

    # def test_find_image(self):
    #     image = self._get_non_test_image()
    #     self.assertIsNotNone(image)
    #     sot = self.conn.compute.find_image(image.id)
    #     self.assertEqual(image.id, sot.id)
    #     self.assertEqual(image.name, sot.name)
    #
    # def test_get_image(self):
    #     image = self._get_non_test_image()
    #     self.assertIsNotNone(image)
    #     sot = self.conn.compute.get_image(image.id)
    #     self.assertEqual(image.id, sot.id)
    #     self.assertEqual(image.name, sot.name)
    #     self.assertIsNotNone(image.links)
    #     self.assertIsNotNone(image.min_disk)
    #     self.assertIsNotNone(image.min_ram)
    #     self.assertIsNotNone(image.metadata)
    #     self.assertIsNotNone(image.progress)
    #     self.assertIsNotNone(image.status)

    def test_image_metadata(self):
        image = "e4b9a8fa-6634-4590-bf9a-8cfc3549f2c1"

        # delete pre-existing metadata
        # self.conn.compute.delete_image_metadata(image, image.metadata.keys())
        image = self.conn.compute.get_image_metadata(image)
        self.assertFalse(image.metadata)

        # # get metadata
        # image = self.conn.compute.get_image_metadata(image)
        # self.assertFalse(image.metadata)
        #
        # # set no metadata
        # self.conn.compute.set_image_metadata(image)
        # image = self.conn.compute.get_image_metadata(image)
        # self.assertFalse(image.metadata)
        #
        # # set empty metadata
        # self.conn.compute.set_image_metadata(image, k0='')
        # image = self.conn.compute.get_image_metadata(image)
        # self.assertFalse(image.metadata)
        #
        # # set metadata
        # self.conn.compute.set_image_metadata(image, k1='v1')
        # image = self.conn.compute.get_image_metadata(image)
        # self.assertTrue(image.metadata)
        # self.assertEqual(1, len(image.metadata))
        # self.assertIn('k1', image.metadata)
        # self.assertEqual('v1', image.metadata['k1'])
        #
        # # set more metadata
        # self.conn.compute.set_image_metadata(image, k2='v2')
        # image = self.conn.compute.get_image_metadata(image)
        # self.assertTrue(image.metadata)
        # self.assertEqual(2, len(image.metadata))
        # self.assertIn('k1', image.metadata)
        # self.assertEqual('v1', image.metadata['k1'])
        # self.assertIn('k2', image.metadata)
        # self.assertEqual('v2', image.metadata['k2'])
        #
        # # update metadata
        # self.conn.compute.set_image_metadata(image, k1='v1.1')
        # image = self.conn.compute.get_image_metadata(image)
        # self.assertTrue(image.metadata)
        # self.assertEqual(2, len(image.metadata))
        # self.assertIn('k1', image.metadata)
        # self.assertEqual('v1.1', image.metadata['k1'])
        # self.assertIn('k2', image.metadata)
        # self.assertEqual('v2', image.metadata['k2'])
        #
        # # delete metadata
        # self.conn.compute.delete_image_metadata(image, image.metadata.keys())
        # image = self.conn.compute.get_image_metadata(image)
        # self.assertFalse(image.metadata)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from ecl.block_store.v2 import _proxy
from ecl.block_store.v2 import snapshot
from ecl.block_store.v2 import type
from ecl.block_store.v2 import volume
from ecl.tests.unit import test_proxy_base


class TestVolumeProxy(test_proxy_base.TestProxyBase):
    def setUp(self):
        super(TestVolumeProxy, self).setUp()
        self.proxy = _proxy.Proxy(self.session)

    def test_snapshot_get(self):
        self.verify_get(self.proxy.get_snapshot, snapshot.Snapshot)

    def test_snapshot_create_attrs(self):
        self.verify_create(self.proxy.create_snapshot, snapshot.Snapshot)

    def test_snapshot_delete(self):
        self.verify_delete(self.proxy.delete_snapshot,
                           snapshot.Snapshot, False)

    def test_snapshot_delete_ignore(self):
        self.verify_delete(self.proxy.delete_snapshot,
                           snapshot.Snapshot, True)

    def test_type_get(self):
        self.verify_get(self.proxy.get_type, type.Type)

    def test_type_create_attrs(self):
        self.verify_create(self.proxy.create_type, type.Type)

    def test_type_delete(self):
        self.verify_delete(self.proxy.delete_type, type.Type, False)

    def test_type_delete_ignore(self):
        self.verify_delete(self.proxy.delete_type, type.Type, True)

    def test_volume_get(self):
        self.verify_get(self.proxy.get_volume, volume.Volume)

    def test_volume_create_attrs(self):
        self.verify_create(self.proxy.create_volume, volume.Volume)

    def test_volume_delete(self):
        self.verify_delete(self.proxy.delete_volume, volume.Volume, False)

    def test_volume_delete_ignore(self):
        self.verify_delete(self.proxy.delete_volume, volume.Volume, True)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from ecl.compute.v2 import _proxy
from ecl.compute.v2 import availability_zone as az
from ecl.compute.v2 import extension
from ecl.compute.v2 import flavor
from ecl.compute.v2 import image
from ecl.compute.v2 import keypair
from ecl.compute.v2 import limits
from ecl.compute.v2 import server
from ecl.compute.v2 import server_interface
from ecl.tests.unit import test_proxy_base2


class TestComputeProxy(test_proxy_base2.TestProxyBase):
    def setUp(self):
        super(TestComputeProxy, self).setUp()
        self.proxy = _proxy.Proxy(self.session)

    def test_extensions(self):
        self.verify_list_no_kwargs(self.proxy.extensions, extension.Extension,
                                   paginated=False)

    def test_flavor_find(self):
        self.verify_find(self.proxy.find_flavor, flavor.Flavor)

    def test_flavor_get(self):
        self.verify_get(self.proxy.get_flavor, flavor.Flavor)

    def test_flavors_detailed(self):
        self.verify_list(self.proxy.flavors, flavor.FlavorDetail,
                         paginated=True,
                         method_kwargs={"details": True, "query": 1},
                         expected_kwargs={"query": 1})

    def test_flavors_not_detailed(self):
        self.verify_list(self.proxy.flavors, flavor.Flavor,
                         paginated=True,
                         method_kwargs={"details": False, "query": 1},
                         expected_kwargs={"query": 1})

    def test_image_delete(self):
        self.verify_delete(self.proxy.delete_image, image.Image, False)

    def test_image_delete_ignore(self):
        self.verify_delete(self.proxy.delete_image, image.Image, True)

    def test_image_find(self):
        self.verify_find(self.proxy.find_image, image.Image)

    def test_image_get(self):
        self.verify_get(self.proxy.get_image, image.Image)

    def test_images_detailed(self):
        self.verify_list(self.proxy.images, image.ImageDetail,
                         paginated=True,
                         method_kwargs={"details": True, "query": 1},
                         expected_kwargs={"query": 1})

    def test_images_not_detailed(self):
        self.verify_list(self.proxy.images, image.Image,
                         paginated=True,
                         method_kwargs={"details": False, "query": 1},
                         expected_kwargs={"query": 1})

    def test_keypair_create(self):
        self.verify_create(self.proxy.create_keypair, keypair.Keypair)

    def test_keypair_delete(self):
        self.verify_delete(self.proxy.delete_keypair, keypair.Keypair, False)

    def test_keypair_delete_ignore(self):
        self.verify_delete(self.proxy.delete_keypair, keypair.Keypair, True)

    def test_keypair_find(self):
        self.verify_find(self.proxy.find_keypair, keypair.Keypair)

    def test_keypair_get(self):
        self.verify_get(self.proxy.get_keypair, keypair.Keypair)

    def test_keypairs(self):
        self.verify_list_no_kwargs(self.proxy.keypairs, keypair.Keypair,
                                   paginated=False)

    def test_limits_get(self):
        self.verify_get(self.proxy.get_limits, limits.Limits, value=[])

    def test_server_interface_create(self):
        self.verify_create(self.proxy.create_server_interface,
                           server_interface.ServerInterface,
                           method_kwargs={"server": "test_id"},
                           expected_kwargs={"server_id": "test_id"})

    def test_server_interface_delete(self):
        self.proxy._get_uri_attribute = lambda *args: args[1]

        interface_id = "test_interface_id"
        server_id = "test_server_id"
        test_interface = server_interface.ServerInterface(id=interface_id)
        test_interface.server_id = server_id

        # Case1: ServerInterface instance is provided as value
        self._verify2("ecl.proxy2.BaseProxy._delete",
                      self.proxy.delete_server_interface,
                      method_args=[test_interface],
                      method_kwargs={"server": server_id},
                      expected_args=[server_interface.ServerInterface],
                      expected_kwargs={"server_id": server_id,
                                       "port_id": interface_id,
                                       "ignore_missing": True})

        # Case2: ServerInterface ID is provided as value
        self._verify2("ecl.proxy2.BaseProxy._delete",
                      self.proxy.delete_server_interface,
                      method_args=[interface_id],
                      method_kwargs={"server": server_id},
                      expected_args=[server_interface.ServerInterface],
                      expected_kwargs={"server_id": server_id,
                                       "port_id": interface_id,
                                       "ignore_missing": True})

    def test_server_interface_delete_ignore(self):
        self.proxy._get_uri_attribute = lambda *args: args[1]
        self.verify_delete(self.proxy.delete_server_interface,
                           server_interface.ServerInterface, True,
                           method_kwargs={"server": "test_id"},
                           expected_args=[server_interface.ServerInterface],
                           expected_kwargs={"server_id": "test_id",
                                            "port_id": "resource_or_id"})

    def test_server_interfaces(self):
        self.verify_list(self.proxy.server_interfaces,
                         server_interface.ServerInterface,
                         paginated=False, method_args=["test_id"],
                         expected_kwargs={"server_id": "test_id"})

    def test_server_create_attrs(self):
        self.verify_create(self.proxy.create_server, server.Server)

    def test_server_delete(self):
        self.verify_delete(self.proxy.delete_server, server.Server, False)

    def test_server_delete_ignore(self):
        self.verify_delete(self.proxy.delete_server, server.Server, True)

    def test_server_force_delete(self):
        self._verify("ecl.compute.v2.server.Server.force_delete",
                     self.proxy.delete_server,
                     method_args=["value", False, True])

    def test_server_find(self):
        self.verify_find(self.proxy.find_server, server.Server)

    def test_server_get(self):
        self.verify_get(self.proxy.get_server, server.Server)

    def test_servers_detailed(self):
        self.verify_list(self.proxy.servers, server.ServerDetail,
                         paginated=True,
                         method_kwargs={"details": True,
                                        "changes_since": 1, "image": 2},
                         expected_kwargs={"changes_since": 1, "image": 2})

    def test_servers_not_detailed(self):
        self.verify_list(self.proxy.servers, server.Server,
                         paginated=True,
                         method_kwargs={"details": False,
                                        "changes_since": 1, "image": 2},
                         expected_kwargs={"paginated": True,
                                          "changes_since": 1, "image": 2})

    def test_server_update(self):
        self.verify_update(self.proxy.update_server, server.Server)

    def test_server_wait_for(self):
        value = server.Server(id='1234')
        self.verify_wait_for_status(
            self.proxy.wait_for_server,
            method_args=[value],
            expected_args=[value, 'ACTIVE', ['ERROR'], 2, 120])

    def test_server_resize(self):
        self._verify("ecl.compute.v2.server.Server.resize",
                     self.proxy.resize_server,
                     method_args=["value", "test-flavor"],
                     expected_args=["test-flavor"])

    def test_availability_zones(self):
        self.verify_list_no_kwargs(self.proxy.availability_zones,
                                   az.AvailabilityZone,
                                   paginated=False)

    def test_get_all_server_metadata(self):
        self._verify2("ecl.compute.v2.server.Server.get_metadata",
                      self.proxy.get_server_metadata,
                      method_args=["value"],
                      method_result=server.Server(id="value", metadata={}),
                      expected_args=[self.session],
                      expected_result={})

    def test_set_server_metadata(self):
        kwargs = {"a": "1", "b": "2"}
        id = "an_id"
        self._verify2("ecl.compute.v2.server.Server.set_metadata",
                      self.proxy.set_server_metadata,
                      method_args=[id],
                      method_kwargs=kwargs,
                      method_result=server.Server.existing(id=id,
                                                           metadata=kwargs),
                      expected_args=[self.session],
                      expected_kwargs=kwargs,
                      expected_result=kwargs)

    def test_delete_server_metadata(self):
        self._verify2("ecl.compute.v2.server.Server.delete_metadata",
                      self.proxy.delete_server_metadata,
                      expected_result=None,
                      method_args=["value", "key"],
                      expected_args=[self.session, "key"])



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from ecl.compute import compute_service
from ecl import resource2


class AbsoluteLimits(resource2.Resource):

    #: The number of key-value pairs that can be set as image metadata.
    image_meta = resource2.Body("maxImageMeta",  alternate_id=True)
    #: The maximum number of personality contents that can be supplied.
    personality = resource2.Body("maxPersonality")
    #: The maximum size, in bytes, of a personality.
    personality_size = resource2.Body("maxPersonalitySize")
    #: The maximum amount of security group rules allowed.
    security_group_rules = resource2.Body("maxSecurityGroupRules")
    #: The maximum amount of security groups allowed.
    security_groups = resource2.Body("maxSecurityGroups")
    #: The amount of security groups currently in use.
    security_groups_used = resource2.Body("totalSecurityGroupsUsed")
    #: The number of key-value pairs that can be set as sever metadata.
    server_meta = resource2.Body("maxServerMeta")
    #: The maximum amount of cores.
    cores = resource2.Body("maxTotalCores")
    #: The amount of cores currently in use.
    cores_used = resource2.Body("totalCoresUsed")
    #: The maximum amount of floating IPs.
    floating_ips = resource2.Body("maxTotalFloatingIps")
    #: The amount of floating IPs currently in use.
    floating_ips_used = resource2.Body("totalFloatingIpsUsed")
    #: The maximum amount of instances.
    instances = resource2.Body("maxTotalInstances")
    #: The amount of instances currently in use.
    instances_used = resource2.Body("totalInstancesUsed")
    #: The maximum amount of keypairs.
    keypairs = resource2.Body("maxTotalKeypairs")
    #: The maximum RAM size in megabytes.
    ram = resource2.Body("maxTotalRAMSize")
    #: The RAM size in megabytes currently in use.
    ram_used = resource2.Body("totalRAMUsed")
    #: The maximum amount of server groups.
    server_groups = resource2.Body("maxServerGroups")
    #: The amount of server groups currently in use.
    server_groups_used = resource2.Body("totalServerGroupsUsed")
    #: The maximum number of members in a server group.
    server_group_members = resource2.Body("maxServerGroupMembers")

    #: New missing attributes
    snapshots = resource2.Body("maxTotalSnapshots")
    snapshots_used = resource2.Body("totalSnapshotsUsed")
    backups = resource2.Body("maxTotalBackups")
    backups_used = resource2.Body("totalBackupsUsed")
    volumes = resource2.Body("maxTotalVolumes")
    volumes_used = resource2.Body("totalVolumesUsed")
    volume_gigabytes = resource2.Body("maxTotalVolumeGigabytes")
    backup_gigabytes = resource2.Body("maxTotalBackupGigabytes")
    backup_gigabytes_used = resource2.Body("totalBackupGigabytesUsed")
    gigabytes_used = resource2.Body("totalGigabytesUsed")


class RateLimit(resource2.Resource):

    #: A list of the specific limits that apply to the ``regex`` and ``uri``.
    limits = resource2.Body("limit", type=list)
    #: A regex representing which routes this rate limit applies to.
    regex = resource2.Body("regex")
    #: A URI representing which routes this rate limit applies to.
    uri = resource2.Body("uri")


class Limits(resource2.Resource):
    base_path = "/limits"
    resource_key = "limits"
    service = compute_service.ComputeService()

    allow_get = True

    absolute = resource2.Body("absolute", type=AbsoluteLimits)
    rate = resource2.Body("rate", type=list)

    def get(self, session, requires_id=False):
        """Get the Limits resource.

        :param session: The session to use for making this request.
        :type session: :class:`~ecl.session.Session`

        :returns: A Limits instance
        :rtype: :class:`~ecl.compute.v2.limits.Limits`
        """
        request = self._prepare_request(requires_id=False, prepend_key=False)

        response = session.get(request.uri, endpoint_filter=self.service)

        body = response.json()
        body = body[self.resource_key]

        absolute_body = self._filter_component(
            body["absolute"], AbsoluteLimits._body_mapping())
        self.absolute = AbsoluteLimits.existing(**absolute_body)

        rates_body = body["rate"]

        rates = []
        for rate_body in rates_body:
            rate_body = self._filter_component(rate_body,
                                               RateLimit._body_mapping())
            rates.append(RateLimit(**rate_body))

        self.rate = rates

        return self



# -*- coding: utf-8 -*-

from ecl.security_portal import security_portal_service
from ecl import resource2
from ecl import exceptions
from ecl import utils


class SecurityDevice(resource2.Resource):
    resource_key = "device"
    resources_key = "devices"
    base_path = '/ecl-api/devices'
    service = security_portal_service.SecurityPortalService()

    _query_mapping = resource2.QueryParameters(
        "tenantid",
        "usertoken",
    )

    # Capabilities
    allow_get = True
    allow_list = True

    # Properties
    #: MSA Device External reference.
    msa_device_id = resource2.Body('msa_device_id', alternate_id=True)
    #: MSA Device Type.
    msa_device_type = resource2.Body('msa_device_type')
    #: Server id of Network-based Security devices.
    os_server_id = resource2.Body('os_server_id')
    #: Server name on Openstack.
    os_server_name = resource2.Body('os_server_name')
    #: Availability zone information.
    os_availability_zone = resource2.Body('os_availability_zone')
    #: Name of admin.
    os_admin_username = resource2.Body('os_admin_username')
    #: Server Status.
    os_server_status = resource2.Body('os_server_status')
    #: Interfaces details associated with the Security Device.
    interfaces = resource2.Body('interfaces')

    def get(self, session, server_id):
        uri = self.base_path + '/%s?tenantid=%s&usertoken=%s' \
              % (server_id, session.get_project_id(), session.get_token())
        resp = session.get(uri, endpoint_filter=self.service)
        self._translate_response(resp, has_body=True)
        return self



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from ecl import proxy2
from ecl import session
from ecl.storage.v1 import availability_zone as _availability_zone
from ecl.storage.v1 import storage as _storage
from ecl.storage.v1 import volume as _volume
from ecl.storage.v1 import volume_type as _volume_type
from ecl.storage.v1 import snapshot as _snapshot
from ecl.storage import version as _version


class VersionSession(session.Session):

    def get_endpoint(self, auth=None, interface='public', service_type=None,
                     **kwargs):
        """Override get endpoint to automate endpoint filtering
        This method uses the service catalog to find the root URI of
        each service and then gets all available versions directly
        from the service, not from the service catalog.
        Endpoints are cached per service type and interface combination
        so that they're only requested from the remote service once
        per instance of this class.
        """

        sc_endpoint = super(VersionSession, self).get_endpoint(
            auth, interface, service_type
        )

        if sc_endpoint is None or sc_endpoint == '':
            return sc_endpoint
        segments = sc_endpoint.split('/')
        if len(segments) > 0:
            segments = segments[:-2]
            sc_endpoint = ""
            for segment in segments:
                sc_endpoint += segment + '/'
            sc_endpoint = sc_endpoint[:-1]

        return sc_endpoint


class Proxy(proxy2.BaseProxy):

    def availability_zones(self, details=True):
        """Return a list of availability zones

        :param bool details: When ``True``, returns
            :class:`~ecl.storage.v1.availability_zone.AvailabilityZoneDetail`
            objects, otherwise :class:`~ecl.storage.v1.availability_zone.AvailabilityZone`.
        :returns: A list of availability zone objects
        """
        availability_zone = _availability_zone.AvailabilityZoneDetail() if details else _availability_zone.AvailabilityZone()
        return list(availability_zone.list(self.session))

    def find_availability_zone(self, name_or_id, ignore_missing=False):
        """Find a single availability_zone

        :param name_or_id: The name or ID of a availability_zone.
        :param bool ignore_missing: When set to ``False``
                                    :class:`~ecl.exceptions.ResourceNotFound` will be
                                    raised when the resource does not exist.
                                    When set to ``True``, None will be returned when
                                    attempting to find a nonexistent resource.
        :returns: One :class:`~ecl.storage.v1.availability_zone.AvailabilityZone` or None
        """
        return self._find(_availability_zone.AvailabilityZone, name_or_id,
                          ignore_missing=ignore_missing)

    def volume_types(self, details=True):
        """Return a generator of volume types

        :param bool details: When ``True``, returns
                             :class:`~ecl.storage.v1.volume_type.VolumeTypeDetail` objects,
                             otherwise :class:`~ecl.storage.v1.volume_type.VolumeType`.
                             *Default: ``True``*
        :returns: A generator of volume type objects
        """
        volume_type = _volume_type.VolumeTypeDetail if details else _volume_type.VolumeType
        return list(self._list(volume_type, paginated=False))

    def find_volume_type(self, name_or_id, ignore_missing=False):
        """Find a volume type

            :param string name_or_id: The name or ID of a volume type.
            :param bool ignore_missing: When set to ``False``
                        :class:`~ecl.exceptions.ResourceNotFound` will be
                        raised when the resource does not exist.
                        When set to ``True``, None will be returned when
                        attempting to find a nonexistent resource.
            :returns: :class:`~ecl.storage.v1.volume_type.VolumeType` or None
            """
        return self._find(_volume_type.VolumeType, name_or_id,
                          ignore_missing=ignore_missing)

    def get_volume_type(self, volume_type_id):
        """Get a single volume type

        :param volume_type_id: The value is the ID of a volume type
        :returns: One :class:`~ecl.storage.v1.volume_type.VolumeType`
                      :class:`~ecl.exceptions.ResourceNotFound`
                      when no resource can be found.
        """
        return self._get(_volume_type.VolumeType, volume_type_id)

    def storages(self, details=True):
        """Return a generator of storages

        :param bool details: When ``True``, returns
                             :class:`~ecl.storage.v1.storage.StorageDetail` objects,
                             otherwise :class:`~ecl.storage.v1.storage.Storage`.
                             *Default: ``True``*
        :returns: A generator of storage objects
        """
        storage = _storage.StorageDetail if details else _storage.Storage
        return list(self._list(storage, paginated=False))

    def get_storage(self, storage_id):
        """Get a single storage

        :param storage_id: The value is the ID of a storage
        :returns: One :class:`~ecl.storage.v1.storage.Storage`
                      :class:`~ecl.exceptions.ResourceNotFound`
                      when no resource can be found.
        """
        return self._get(_storage.Storage, storage_id)

    def create_storage(self, network_id, subnet_id, ip_addr_pool, name,
                       volume_type_id, description=None, host_routes=None):
        """This API create additional Storage.

        :param network_id: Network ID.
        :param subnet_id: Subnetwork ID.
        :param ip_addr_pool: The pool of IP addresses for storage nodes used by the volume.
                             The IP address range may be overwraped with subnet's allocation_pools.
                             The range should be included in CIDR of the subnet.
        :param name: Name of Virtual Storage.
        :param volume_type_id: volume type ID.
        :param description: Description of Virtual Storage.
        :param host_routes: Static routes settings for each virtual storages.
        :return: :class:`~ecl.storage.v1.storage.Storage`
        """
        body = {}
        body["network_id"] = network_id
        body["subnet_id"] = subnet_id
        body["ip_addr_pool"] = ip_addr_pool
        body["name"] = name
        body["volume_type_id"] = volume_type_id
        if description:
            body["description"] = description
        if host_routes:
            body["host_routes"] = host_routes
        storage = _storage.Storage()
        return storage.create(self.session, **body)

    def update_storage(self, storage_id, **body):
        """This API update existing storage

        :param kwargs body: Request parameter to update storage.

            * storage_id: ID for the requested storage.
            * name: Name of Virtual Storage.
            * description: Description of Virtual Storage.
            * ip_addr_pool: The pool of IP addresses for storage nodes used by the volume. The IP address range may be overwraped with subnet's allocation_pools. The range should be included in CIDR of the subnet.
            * host_routes: Static routes settings for each virtual storages.
        :return: :class:`~ecl.storage.v1.storage.Storage`
        """
        storage = _storage.Storage()
        return storage.update(self.session, storage_id, **body)

    def delete_storage(self, storage_id):
        """This API deletes a specified storage.

        :param storage_id: ID for the specified storage.
        :return: ``None``
        """

        return self._delete(_storage.Storage, storage_id)

    def volumes(self, details=True):
        """Return a list of volumes

        :param bool details: When ``True``, returns
                             :class:`~ecl.storage.v1.volume_type.VolumeTypeDetail` objects,
                             otherwise :class:`~ecl.storage.v1.volume_type.VolumeType`.
                             *Default: ``True``*
        :returns: A generator of volume type objects
        """
        volume = _volume.VolumeDetail if details else _volume.Volume
        return list(self._list(volume, paginated=False))

    def get_volume(self, volume_id):
        """Get a single volume

        :param volume_id: The value is the ID of a volume
        :returns: One :class:`~ecl.storage.v1.volume.Volume`
                      :class:`~ecl.exceptions.ResourceNotFound`
                      when no resource can be found.
        """
        return self._get(_volume.Volume, volume_id)

    def create_volume(self, virtual_storage_id, name, size, description=None,
                      iops_per_gb=None, initiator_iqns=None, throughput=None,
                      availability_zone=None):
        """This API create additional Volume.

        :param virtual_storage_id: Virtual Storage ID.
        :param name: Name of volume.
        :param size: Size of volume in Gigabyte.
        :param description: Description of volume.
        :param iops_per_gb: Provisioned IOPS/GB for volume.
        :param throughput: Provisioned throughput for volume.
        :param initiator_iqns: List of initiator IQN who can access to this volume.
        :param availability_zone: Availability zone.
        :return: :class:`~ecl.storage.v1.volume.Volume`
        """
        body = {}
        body["virtual_storage_id"] = virtual_storage_id
        body["name"] = name
        body["size"] = size
        if description:
            body["description"] = description
        if iops_per_gb:
            body["iops_per_gb"] = iops_per_gb
        if throughput:
            body["throughput"] = throughput
        if initiator_iqns:
            body["initiator_iqns"] = initiator_iqns
        if availability_zone:
            body["availability_zone"] = availability_zone
        volume = _volume.Volume()
        return volume.create(self.session, **body)

    def update_volume(self, volume_id, **body):
        """This API update existing volume.

        :param kwargs body: Request parameter to update volume.

            * volume_id: ID for the requested volume.
            * name: Name of volume.
            * description: Description of volume.
            * initiator_iqns: List of initiator IQN who can access to this volume.
        :return: :class:`~ecl.storage.v1.volume.Volume`
        """
        volume = _volume.Volume()
        return volume.update(self.session, volume_id, **body)

    def delete_volume(self, volume_id):
        """This API deletes a specified volume.

        :param  volume_id: ID for the specified volume.
        :return: ``None``
        """

        return self._delete(_volume.Volume, volume_id)

    def snapshots(self, details=True, **params):
        """Return a list of snapshots.

        :param bool details: When ``True``, returns
                             :class:`~ecl.storage.v1.snapshot.SnapshotDetail` objects,
                             otherwise :class:`~ecl.storage.v1.snapshot.Snapshot`.
                             *Default: ``True``*
        :param kwargs params: parameter used as query parameter
                              for GET request.
        :returns: A list of snapshot objects
        """
        snapshot = _snapshot.SnapshotDetail if details else _snapshot.Snapshot
        return list(self._list(snapshot, paginated=False, **params))

    def get_snapshot(self, snapshot_id):
        """Get a single snapshot.

        :param snapshot_id: The value is the ID of a snapshot
        :returns: One :class:`~ecl.storage.v1.snapshot.Snapshot`
                      :class:`~ecl.exceptions.ResourceNotFound`
                      when no resource can be found.
        """
        return self._get(_snapshot.Snapshot, snapshot_id)

    def create_snapshot(self, volume_id, name, description=None):
        """This API create snapshot of volume.

        :param volume_id: Parent volume ID.
        :param name: Name of snapshot.
        :param description: Description of snapshot.
        :return: :class:`~ecl.storage.v1.snapshot.Snapshot`
        """
        body = {}
        body["volume_id"] = volume_id
        body["name"] = name
        if description:
            body["description"] = description
        snapshot = _snapshot.Snapshot()
        return snapshot.create(self.session, **body)

    def update_snapshot(self, snapshot_id, **body):
        """This API update existing snapshot.

        :param snapshot_id: ID for the requested snapshot.
        :param name: Name of snapshot.
        :param description: Description of snapshot.
        :return: :class:`~ecl.storage.v1.snapshot.Snapshot`
        """
        snapshot = _snapshot.Snapshot()
        return snapshot.update(self.session, snapshot_id, **body)

    def delete_snapshot(self, snapshot_id):
        """This API deletes a specified snapshot.

        :param  snapshot_id: ID for the specified snapshot.
        :return: ``None``
        """
        return self._delete(_snapshot.Snapshot, snapshot_id)

    def restore_snapshot(self, snapshot_id):
        """Restore volume from snapshot.

        :param snapshot_id: Snapshot ID to utilize as restore source.
        :return: <Response 202>
        """
        snapshot = self.get_snapshot(snapshot_id)
        return snapshot.restore(self.session)

    def get_version(self):
        """
        Show Storage API version.

        :return: :class:`~ecl.storage.version.Version`
        """

        version = _version.Version()
        v_session = VersionSession(
            profile=self.session.profile,
            user_agent=self.session.user_agent,
        )
        for attr, value in self.session.__dict__.iteritems():
            v_session.__setattr__(attr, value)

        return version.get_version(session=v_session)

    def versions(self):
        """
        List Storage API version.

        :return: A generator of version objects
        """

        version = _version.Version()
        v_session = VersionSession(
            profile=self.session.profile,
            user_agent=self.session.user_agent,
        )
        for attr, value in self.session.__dict__.iteritems():
            v_session.__setattr__(attr, value)

        return list(version.list_version(session=v_session))



# -*- coding: utf-8 -*-


from ecl.network import network_service
from ecl.network.v2.base import NetworkBaseResource
from ecl import resource2


class PublicIP(NetworkBaseResource):

    resource_key = "public_ip"
    resources_key = "public_ips"
    service = network_service.NetworkService("v2.0")
    base_path = '/' + service.version + '/public_ips'

    _query_mapping = resource2.QueryParameters(
        "cidr", "description", "id", "internet_gw_id",
        "name", "status", "submask_length", "tenant_id",
        "sort_key", "sort_dir",
    )

    allow_list = True
    allow_get = True
    allow_create = True
    allow_delete = True
    allow_update = True

    cidr = resource2.Body("cidr")
    description = resource2.Body("description")
    id = resource2.Body("id")
    internet_gw_id = resource2.Body("internet_gw_id")
    name = resource2.Body("name")
    status = resource2.Body("status")
    submask_length = resource2.Body("submask_length", int)
    tenant_id = resource2.Body("tenant_id")

    def wait_for_publicip(self, session, status='ACTIVE', failures=['ERROR'],
                          interval=2, wait=120):
        return resource2.wait_for_status(session, self, status,
                                         failures, interval, wait)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import six
from ecl.tests.functional import base

class TestUser(base.BaseFunctionalTest):

    @classmethod
    def test_01_list_user(cls):
        users = list(cls.conn.rca.list_users())
        user = users[0]
        cls.user_name = user.id
        assert isinstance(user.id, six.string_types)
        assert isinstance(user.vpn_endpoints, list)

    def test_02_show_user(self):
        user = self.conn.rca.get_user(self.user_name)
        self.assertIsInstance(user.name, six.string_types)
        self.assertIsInstance(user.vpn_endpoints, list)

    def test_03_create_user(self):
        user = self.conn.rca.create_user(
            password="Passw0rd!"
        )

    def test_04_update_user(self):
        user = self.conn.rca.update_user(
            username="72ab9350a47a4173966ad02dc51b32a1", password="Pa55w0rd+"
        )
        self.assertIsInstance(user.name, six.string_types)
        self.assertIsInstance(user.vpn_endpoints, list)

    def test_05_delete_user(self):
        self.conn.rca.delete_user("wrong_user_name")







# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from ecl import resource
from ecl.telemetry import telemetry_service
from ecl import utils


class Alarm(resource.Resource):
    """.. caution:: This API is a work in progress and is subject to change."""
    id_attribute = 'alarm_id'
    base_path = '/alarms'
    service = telemetry_service.TelemetryService()

    # Supported Operations
    allow_create = True
    allow_retrieve = True
    allow_update = True
    allow_delete = True
    allow_list = True

    # Properties
    #: The actions to do when alarm state changes to alarm
    alarm_actions = resource.prop('alarm_actions')
    #: The ID of the alarm
    alarm_id = resource.prop('alarm_id')
    # TODO(briancurtin): undocumented
    combination_rule = resource.prop('combination_rule')
    #: The description of the alarm
    description = resource.prop('description')
    #: ``True`` if this alarm is enabled. *Type: bool*
    is_enabled = resource.prop('enabled', type=bool)
    #: The actions to do when alarm state changes to insufficient data
    insufficient_data_actions = resource.prop('insufficient_data_actions')
    #: The actions should be re-triggered on each evaluation cycle.
    #: *Type: bool*
    is_repeat_actions = resource.prop('repeat_actions', type=bool)
    #: The name for the alarm
    name = resource.prop('name')
    #: The actions to do when alarm state change to ok
    ok_actions = resource.prop('ok_actions')
    #: The ID of the project that owns the alarm
    project_id = resource.prop('project_id')
    #: The severity of the alarm
    severity = resource.prop('severity')
    #: The state off the alarm
    state = resource.prop('state')
    #: The timestamp of the last alarm state change.
    state_changed_at = resource.prop('state_timestamp')
    # TODO(briancurtin): undocumented
    threshold_rule = resource.prop('threshold_rule', type=dict)
    #: Describe time constraints for the alarm
    time_constraints = resource.prop('time_constraints')
    #: Explicit type specifier to select which rule to follow
    type = resource.prop('type')
    #: The timestamp of the last alarm definition update.
    updated_at = resource.prop('timestamp')
    #: The ID of the user who created the alarm
    user_id = resource.prop('user_id')

    def change_state(self, session, next_state):
        """Set the state of an alarm.

           The next_state may be one of: 'ok' 'insufficient data' 'alarm'
        """
        url = utils.urljoin(self.base_path, self.id, 'state')
        resp = session.put(url, endpoint_filter=self.service, json=next_state)
        return resp.json()

    def check_state(self, session):
        """Retrieve the current state of an alarm from the service.

           The properties of the alarm are not modified.
        """
        url = utils.urljoin(self.base_path, self.id, 'state')
        resp = session.get(url, endpoint_filter=self.service)
        resp = resp.json()
        current_state = resp.replace('\"', '')
        return current_state



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import six
from ecl.tests.functional import base

class TestMember(base.BaseFunctionalTest):
    image_id = "1b782d43-4e43-4d96-883b-ff423f8b8b7c"

    def test_01_list(self):
        members = list(self.conn.image.members(self.image_id))
        print members

    def test_02_create_member(self):
        member = self.conn.image.create_member(
            self.image_id,
            member="f6a818c3d4aa458798ed86892e7150c0"
        )
        print member

    def test_03_update_member(self):
        member = self.conn.image.update_member(
            self.image_id,
            "f6a818c3d4aa458798ed86892e7150c0",
            status="accepted"
        )
        print member

    def test_04_show_member(self):
        member = self.conn.image.get_member(
            self.image_id, "f6a818c3d4aa458798ed86892e7150c0"
        )
        print member
        self.assertIsInstance(member.member_id, six.string_types)
        self.assertIsInstance(member.created_at, six.string_types)
        self.assertIsInstance(member.image_id, six.string_types)
        self.assertIsInstance(member.status, six.string_types)
        self.assertIsInstance(member.schema, six.string_types)
        self.assertIsInstance(member.updated_at, six.string_types)

    def test_05_delete_member(self):
        member = self.conn.image.delete_member(
            self.image_id, "f6a818c3d4aa458798ed86892e7150c0"
        )


# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from ecl.dedicated_hypervisor.v1 import license as _license
from ecl.dedicated_hypervisor.v1 import license_type as _license_type
from ecl.dedicated_hypervisor.v1 import server as _server
from ecl.dedicated_hypervisor.v1 import usage as _usage
from ecl.dedicated_hypervisor.v1 import vcf as _vcf
from ecl import proxy2


class Proxy(proxy2.BaseProxy):

    def servers(self, details=True, **query):
        """
        List your Dedicated Hypervisor servers information.

        :param query: Query parameters to select results
        :return: list of the servers
        :rtype: list of :class:`~ecl.dedicated_hypervisor.v1.server.Server`
        """
        server = _server.ServerDetail if details else _server.Server
        return list(self._list(server, **query))

    def get_server(self, server_id):
        """
        Shows your Dedicated Hypervisor server information.

        :param string server_id: ID for the server.
        :return: One :class:`~ecl.dedicated_hypervisor.v1.server.Sever`
            instance.
        :rtype: :class:`~ecl.dedicated_hypervisor.v1.server.Server`
        """
        return self._get(_server.Server, server_id)

    def create_server(self, name, networks, image, flavor,
                      admin_pass=None, description=None, availability_zone=None,
                      metadata=None):
        """
        Create additional Dedicated Hypervisor server.

        :param string name: Name of your Dedicated Hypervisor/Baremetal server
            as a string.
        :param string description: Description of your Dedicated Hypervisor
            server as a string.
        :param array networks: If it is specified greater than two,
            default gateway is first network.
        :param string flavor: The flavor reference for the desired flavor
            for your Dedicated Hypervisor server.
            Specify as an UUID or full URL.
            Parameters of id or links -> href in api_list_flavors page
            can be used as flavorRef parameter.
        :param string admin_pass: Password for the administrator.
        :param string image: The image reference for the desired image
            for your Dedicated Hypervisor server.
            Specify as an UUID or full URL.
        :param string availability_zone: The availability zone name
            in which to launch the server.
            If omit this parameter, target availability_zone is random.
        :param dict metadata: Metadata key and value pairs.
            The maximum size of the metadata key and value is 255 bytes each.
        :return: The results of server createion.
        :rtype: :class:`~ecl.dedicated_hypervisor.v1.server.Server`
        """
        body = {}
        body["name"] = name
        body["networks"] = networks
        body["imageRef"] = image
        body["flavorRef"] = flavor
        if admin_pass:
            body["adminPass"] = admin_pass
        if description:
            body["description"] = description
        if availability_zone:
            body["availability_zone"] = availability_zone
        if metadata:
            body["metadata"] = metadata
        return self._create(_server.Server, **body)

    def delete_server(self, server_id, ignore_missing=False):
        """
        Deletes a specified Dedicated Hypervisor server.
        You can delete server when specified server status is
        ACTIVE or ERROR status.

        :param string server_id: ID for the server.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the server does not exist.
                    When set to ``True``, no exception will be set when
                    attempting to delete a nonexistent server
        :returns: ``None``
        """
        return self._delete(_server.Server, server_id,
                            ignore_missing=ignore_missing)

    def license_types(self):
        """
        Lists your Guest Image license on Dedicated Hypervisor servers
        information.

        :return: list of the license types.
        :rtype: list of
            :class:`~ecl.dedicated_hypervisor.v1.license_type.LicenseType`
        """
        return list(self._list(_license_type.LicenseType, paginated=False))

    def find_license_type(self, name_or_id, ignore_missing=False):
        """Find a single license type

        :param name_or_id: The name or ID of a license type.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the resource does not exist.
                    When set to ``True``, None will be returned when
                    attempting to find a nonexistent resource.
        :returns: One :class:`~ecl.dedicated_hypervisor.v1.license_type.LicenseType` or None
        """
        return self._find(_license_type.LicenseType, name_or_id,
                          ignore_missing=ignore_missing)

    def licenses(self, license_type=None):
        """
        Lists your Guest Image license key information.

        :return: list of the licenses.
        :rtype: list of
            :class:`~ecl.dedicated_hypervisor.v1.license.License`
        """
        return list(self._list(_license.License, license_type=license_type,
                               paginated=False))

    def create_license(self, license_type):
        """
        Create additional Guest Image license key.
        You can create additional license key only for the licenses which
        `has_license_key` property is True.

        :param string license_type: Name of your Guest Image license type
            as a string.
        :return: The results of license createion.
        :rtype: :class:`~ecl.dedicated_hypervisor.v1.license.License`
        """

        lic = _license.License()
        return lic.create(self.session, license_type)

    def delete_license(self, license_id, ignore_missing=False):
        """
        Delete a specified Guest Image license key.

        :param string license_id: ID for the license
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the server does not exist.
                    When set to ``True``, no exception will be set when
                    attempting to delete a nonexistent server
        :returns: ``None``
        """
        return self._delete(_license.License, license_id,
                            ignore_missing=ignore_missing)

    def usages(self):
        """
        Lists your Guest Image usage information.

        :return: list of the usages
        :rtype: list of :class:`~ecl.dedicated_hypervisor.v1.usage.Usage`
        """
        return list(self._list(_usage.Usage))

    def get_usage_histories(self, history_id, From=None, to=None):
        """
        Shows your Guest Image usage history information.

        :param string history_id: ID for the history.
        :param dateTime From: date to list usage from. `YYYY-MM-DD hh:mm:ss` .
        :param dateTime to: date to list usage to. month of the parameter
            must be the same as `from` . `YYYY-MM-DD hh:mm:ss` .
        :return: :class:`~ecl.dedicated_hypervisor.v1.usage.Usage`
        """
        query = {}
        if From:
            query["from"] = From
        if to:
            query["to"] = to
        usage = _usage.Usage()
        return usage.get_usage_histories(session=self.session,
                                         history_id=history_id, **query)

    def add_license(self, server_id, license_types, vm_id=None, vm_name=None):
        """Add license to VM on hyper visor

        :param string server_id: ID of server that have VM to add license
        :param array license_types: Array of license type's Name
        :param string vm_id: VM ID, Specify either vm_id or vm_name
        :param string vm_name: VM Name, Specify either vm_id or vm_name
        :returns: dict of job_id
        """
        server = _server.ServerAction()
        return server.add_license(self.session, server_id, license_types,
                                  vm_id=vm_id, vm_name=vm_name)

    def get_add_license_job(self, server_id, job_id):
        """Get add license job status

        :param string server_id: Server ID
        :param string job_id: Job ID
        :returns: Job object
        """
        server = _server.ServerAction()
        return server.get_add_license_job(self.session, server_id, job_id)


    def sddcs(self, **query):
        return list(self._list(_vcf.Sddc, **query))


    def delete_sddc(self, sddc_id, ignore_missing=False):
        return self._delete(_vcf.Sddc, sddc_id, ignore_missing=ignore_missing)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import testtools

from ecl.storage.v1 import storage as s

IDENTIFIER = 'IDENTIFIER'
BASIC_EXAMPLE = {
    'id': IDENTIFIER,
    'network_id': '2',
    'subnet_id': '3',
    'ip_addr_pool': {
        'start': '192.168.0.128',
        'end': '192.168.0.255'
    },
    'host_routes': [{
        'destination': '0.0.0.0/0',
        'nexthop': '123.456.78.9'
    },],
    'volume_type_id': '6685584b-1eac-4da6-b5c3-555430cf68ff',
    'name': 'vs01',
    'description': 'Virtual Storage for NW1',
    'status': 'creating',
    'created_at': 'null',
    'updated_at': 'null',
    'error_message': '',
}


class TestStorage(testtools.TestCase):

    def test_basic(self):
        sot = s.Storage()
        self.assertEqual('virtual_storage', sot.resource_key)
        self.assertEqual('virtual_storages', sot.resources_key)
        self.assertEqual('/virtual_storages', sot.base_path)
        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_get)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_update)
        self.assertTrue(sot.allow_delete)
        self.assertEqual('storage', sot.service.service_type)

    def test_basic_detail(self):
        sot = s.StorageDetail()
        self.assertEqual('virtual_storage', sot.resource_key)
        self.assertEqual('virtual_storages', sot.resources_key)
        self.assertEqual('/virtual_storages/detail', sot.base_path)
        self.assertTrue(sot.allow_list)
        self.assertFalse(sot.allow_get)
        self.assertFalse(sot.allow_create)
        self.assertFalse(sot.allow_update)
        self.assertFalse(sot.allow_delete)
        self.assertEqual('storage', sot.service.service_type)

    def test_make_basic(self):
        sot = s.Storage(**BASIC_EXAMPLE)
        self.assertEqual(BASIC_EXAMPLE['id'], sot.id)
        self.assertEqual(BASIC_EXAMPLE['network_id'], sot.network_id)
        self.assertEqual(BASIC_EXAMPLE['subnet_id'], sot.subnet_id)
        self.assertEqual(BASIC_EXAMPLE['ip_addr_pool'], sot.ip_addr_pool)
        self.assertEqual(BASIC_EXAMPLE['host_routes'], sot.host_routes)
        self.assertEqual(BASIC_EXAMPLE['volume_type_id'], sot.volume_type_id)
        self.assertEqual(BASIC_EXAMPLE['name'], sot.name)
        self.assertEqual(BASIC_EXAMPLE['description'], sot.description)
        self.assertEqual(BASIC_EXAMPLE['status'], sot.status)
        self.assertEqual(BASIC_EXAMPLE['created_at'], sot.created_at)
        self.assertEqual(BASIC_EXAMPLE['updated_at'], sot.updated_at)
        self.assertEqual(BASIC_EXAMPLE['error_message'], sot.error_message)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from ecl import resource
from ecl.telemetry import telemetry_service


class AlarmChange(resource.Resource):
    """.. caution:: This API is a work in progress and is subject to change."""
    id_attribute = 'event_id'
    resource_key = 'alarm_change'
    base_path = '/alarms/%(alarm_id)s/history'
    service = telemetry_service.TelemetryService()

    # Supported Operations
    allow_list = True

    # Properties
    #: The ID of the alarm
    alarm_id = resource.prop('alarm_id')
    #: Data describing the change
    detail = resource.prop('detail')
    #: The ID of the change event
    event_id = resource.prop('event_id')
    #: The project ID on behalf of which the change is being made
    on_behalf_of_id = resource.prop('on_behalf_of')
    #: The project ID of the initiating identity
    project_id = resource.prop('project_id')
    #: The time/date of the alarm change.
    triggered_at = resource.prop('timestamp')
    #: The type of change
    type = resource.prop('type')
    #: The user ID of the initiating identity
    user_id = resource.prop('user_id')

    @classmethod
    def list(cls, session, limit=None, marker=None, path_args=None,
             paginated=False, **params):
        url = cls._get_url(path_args)
        resp = session.get(url, endpoint_filter=cls.service, params=params)
        for item in resp.json():
            yield cls.existing(**item)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import six
from ecl.tests.functional import base

class TestImageCopy(base.BaseFunctionalTest):
    image_id = "4ddf938a-2cff-46a8-ab7b-efb48c8f79a7"

    @classmethod
    def test_01_copy_image(cls):
        copy = cls.conn.image.copy_image(
            cls.image_id, "72ab9350a47a4173966ad02dc51b32a1")
        print copy
        cls.job_id = copy.job_id
        assert isinstance(copy.job_id, six.string_types)

    def test_02_list_image_copy_jobs(self):
        jobs = list(self.conn.image.list_image_copy_jobs())
        for job in jobs:
            self.assertIsInstance(job.source_image_id, six.string_types)
            self.assertIsInstance(job.source_region_id, six.string_types)
            self.assertIsInstance(job.source_tenant_id, six.string_types)
            self.assertIsInstance(job.destination_image_id, six.string_types)
            self.assertIsInstance(job.destination_region_id, six.string_types)
            self.assertIsInstance(job.destination_tenant_id, six.string_types)
            self.assertIsInstance(job.status, six.string_types)
            self.assertIsInstance(job.copy_progress, int)
        print jobs

    def test_03_get_image_copy_job(self):
        job = self.conn.image.get_image_copy_job(
            "c02103c2-3b6e-4f5e-9be9-7f6a8d18da26"
        )
        self.assertIsInstance(job.source_image_id, six.string_types)
        self.assertIsInstance(job.source_region_id, six.string_types)
        self.assertIsInstance(job.source_tenant_id, six.string_types)
        self.assertIsInstance(job.destination_image_id, six.string_types)
        self.assertIsInstance(job.destination_region_id, six.string_types)
        self.assertIsInstance(job.destination_tenant_id, six.string_types)
        self.assertIsInstance(job.status, six.string_types)
        self.assertIsInstance(job.copy_progress, int)
        print job

    def test_04_cancel_copy_image(self):
        copy = self.conn.image.cancel_copy_image(
            "c02103c2-3b6e-4f5e-9be9-7f6a8d18da26"
        )
        print copy


# -*- coding: utf-8 -*-

import six
from ecl.tests.functional import base


class TestReservedAddress(base.BaseFunctionalTest):

    @classmethod
    def setUpClass(cls):
        super(TestReservedAddress, cls).setUpClass()
        cls.one_reserved_address = None
        sots = cls.conn.network.reserved_addresses()
        if sots and len(sots) > 0:
            cls.one_reserved_address = sots[0]

    def test_list(self):
        sots = self.conn.network.reserved_addresses()
        for sot in sots:
            self.assertIsInstance(sot.id, six.string_types)
            self.assertIsInstance(sot.subnets, list)
            self.assertIsInstance(sot.tenant_id, six.string_types)

    def test_get(self):
        if not self.one_reserved_address:
            print "0 reserved addresses."
            return
        sot = self.conn.network.get_reserved_address(self.one_reserved_address.id)
        self.assertEqual(sot.id, self.one_reserved_address.id)
        self.assertEqual(sot.subnets, self.one_reserved_address.subnets)
        self.assertEqual(sot.tenant_id, self.one_reserved_address.tenant_id)



# -*- coding: utf-8 -*-

from ecl.network import network_service
from ecl.network.v2.base import NetworkBaseResource
from ecl import resource2
from ecl import exceptions


class AzureService(NetworkBaseResource):

    resource_key = "azure_service"
    resources_key = "azure_services"
    service = network_service.NetworkService("v2.0")
    base_path = '/' + service.version + '/azure_services'

    allow_list = True
    allow_get = True

    description = resource2.Body("description")
    id = resource2.Body("id")
    zone = resource2.Body("zone")
    name = resource2.Body("name")


class AzureGateway(NetworkBaseResource):

    resource_key = "azure_gateway"
    resources_key = "azure_gateways"
    service = network_service.NetworkService("v2.0")
    base_path = '/' + service.version + '/azure_gateways'

    allow_list = True
    allow_get = True

    description = resource2.Body("description")
    id = resource2.Body("id")
    azure_service_id = resource2.Body("azure_service_id")
    name = resource2.Body("name")
    qos_option_id = resource2.Body("qos_option_id")
    status = resource2.Body("status")
    tenant_id = resource2.Body("tenant_id")


class AzureInterface(NetworkBaseResource):

    resource_key = "azure_interface"
    resources_key = "azure_interfaces"
    service = network_service.NetworkService("v2.0")
    base_path = '/' + service.version + '/azure_interfaces'

    allow_list = True
    allow_get = True

    _query_mapping = resource2.QueryParameters(
        "azure_gw_id",
    )

    azure_gw_id = resource2.Body("azure_gw_id")
    description = resource2.Body("description")
    id = resource2.Body("id")
    name = resource2.Body("name")
    status = resource2.Body("status")
    tenant_id = resource2.Body("tenant_id")
    primary = resource2.Body("primary")
    secondary = resource2.Body("secondary")



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import six
from ecl.tests.functional import base

class TestAVZone(base.BaseFunctionalTest):
    def test_01_av_zones(self):
        zones = list(self.conn.storage.availability_zones(details=True))
        print zones[0]
        zone = zones[0]
        self.assertIsInstance(zone.zoneState, dict)
        self.assertIsInstance(zone.zoneName, six.string_types)

    def test_02_find_av_zones(self):
        zones = self.conn.storage.find_availability_zone("zone1-groupa")
        print zones




# -*- coding: utf-8 -*-

import testtools

from ecl.connectivity import connectivity_service


class TestConnectivityService(testtools.TestCase):

    def test_service(self):
        sot = connectivity_service.ConnectivityService()
        self.assertEqual('interconnectivity', sot.service_type)
        self.assertEqual('public', sot.interface)
        self.assertIsNone(sot.region)
        self.assertIsNone(sot.service_name)
        self.assertEqual(1, len(sot.valid_versions))
        self.assertEqual('v1', sot.valid_versions[0].module)
        self.assertEqual('v1', sot.valid_versions[0].path)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import six
from ecl.tests.functional import base

class TestMetadata(base.BaseFunctionalTest):
    server_id = "752aac2e-4b82-4d47-a7c7-fcbd0cbc86e2"

    def setUp(self):
        print "\n>> Test Name: " + self.id()

    def test_01_list(self):
        metadata = self.conn.baremetal.metadata(self.server_id)
        print metadata
        self.assertIsInstance(metadata.metadata, dict)

    def test_02_replace(self):
        metadata = self.conn.baremetal.replace_metadata(
            self.server_id,
            metadata={"KEY1":"VALUE1",
                      "key2":"value2"}
        )
        print metadata
        self.assertIsInstance(metadata.metadata, dict)

    def test_03_delete(self):
        metadata = self.conn.baremetal.delete_metadata(
            self.server_id, "KEY1"
        )


    def test_04_merge(self):
        metadata = self.conn.baremetal.merge_metadata(
            self.server_id,
            metadata={"KEY60":"VALUE60"}
        )
        print metadata
        self.assertIsInstance(metadata.metadata, dict)

    def test_05_show(self):
        metadata = self.conn.baremetal.show_metadata(
            self.server_id, "KEY60"
        )
        print metadata
        self.assertIsInstance(metadata.metadata, dict)

    def test_06_update(self):
        metadata = self.conn.baremetal.update_metadata(
            self.server_id, "KEY60",
            metadata={"KEY60":"NEW_VALUE00"}
        )
        print metadata
        self.assertIsInstance(metadata.metadata, dict)

    def test_07_delete(self):
        metadata = self.conn.baremetal.delete_metadata(
            self.server_id, "key2"
        )

    def test_08_list(self):
        metadata = self.conn.baremetal.metadata(self.server_id)
        print metadata
        self.assertIsInstance(metadata.metadata, dict)





# -*- coding: utf-8 -*-

from ecl.tests.functional import base


class TestQuota(base.BaseFunctionalTest):

    def test_get(self):
        quota_id = "tenant"
        sot = self.conn.network.get_quota(quota=quota_id)
        self.assertIn('colocation_logical_link', sot)
        self.assertIn('common_function_gateway', sot)
        self.assertIn('firewall', sot)
        self.assertIn('interdc_gateway', sot)
        self.assertIn('internet_gateway', sot)
        self.assertIn('load_balancer', sot)
        self.assertIn('network', sot)
        self.assertIn('port', sot)
        self.assertIn('subnet', sot)
        self.assertIn('vpn_gateway', sot)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from ecl.image import image_service
from ecl import resource2

class License(resource2.Resource):
    service = image_service.ImageService()
    base_path = '/' + service.version + '/extension/license_switch/types'

    # Properties
    #: ("WindowsServer_2012R2_Standard_64_ComLicense",
    #: "WindowsServer_2012_Standard_64_ComLicense",
    #: "WindowsServer_2008R2_Enterprise_64_ComLicense",
    #: "WindowsServer_2008R2_Standard_64_ComLicense",
    #: "Red_Hat_Enterprise_Linux_7_64bit_BYOL")
    license_switch_types = resource2.Body('license_switch_types')

    def list(self, session):
        """List image license switch types."""

        resp = session.get(self.base_path,
                           headers={"Accept": "application/json"},
                           endpoint_filter=self.service
                           )
        self._translate_response(resp, has_body=True)
        return self